@@ -19,11 +19,12 @@ import chisel
 action pypi_index
     input
         string package
+        optional bool forceUpdate
 ''')
 def pypi_index(ctx, req):
 
     # Get the package index
-    packageIndex = ctx.index.getPackageIndex(ctx, req['package'])
+    packageIndex = ctx.index.getPackageIndex(ctx, req['package'], req.get('forceUpdate', False))
     if packageIndex is None:
         return ctx.responseText('404 Not Found', 'Not Found')
 
@@ -91,7 +92,9 @@ _uploadFiletypeToExt = {
     'sdist': '.tar.gz'
 }
 
-def _pypi_upload(ctx):
+@chisel.request
+def pypi_upload(environ, start_response):
+    ctx = environ[chisel.Application.ENVIRON_APP]
 
     # Decode the multipart post
     ctype, pdict = cgi.parse_header(ctx.environ.get('CONTENT_TYPE', ''))
@@ -133,16 +136,6 @@ def _pypi_upload(ctx):
         return ctx.responseText('400 Bad Request', 'Bad Request')
 
 
-@chisel.request
-def pypi_upload(environ, start_response):
-    ctx = environ[chisel.Application.ENVIRON_APP]
-    try:
-        return _pypi_upload(ctx)
-    except:
-        ctx.log.exception('Exception raised in pypi_upload')
-        return ctx.responseText('500 Internal Server Error', 'Internal Server Error')
-
-
 #
 # Pypi application
 #
@@ -151,11 +144,15 @@ class MrPyPi(chisel.Application):
     __slots__ = ('index')
 
     def __init__(self, index):
+
+        # Override application defaults
         chisel.Application.__init__(self)
-        self.index = index
         self.logLevel = logging.INFO
 
-    def init(self):
+        # Set the index
+        self.index = index
+
+        # Add application requests
         self.addDocRequest()
         self.addRequest(pypi_index)
         self.addRequest(pypi_download)
